--[[
Copyright (c) 2008, LibQTipClick Development Team 

All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, 
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution.
    * Redistribution of a stand alone version is strictly prohibited without 
      prior written authorization from the Lead of the LibQTipClick Development Team. 
    * Neither the name of the LibQTipClick Development Team nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--]]

assert(LibStub, "LibQTipClick-1.0 requires LibStub")

local MAJOR, MINOR = "LibQTipClick-1.0", 1
local LibQTipClick, oldminor = LibStub:NewLibrary(MAJOR, MINOR)
if not LibQTipClick then return end -- No upgrade needed

local LibQTip = LibStub:GetLibrary("LibQTip-1.0")

assert(LibQTip, "LibQTipClick-1.0 requires LibQTip-1.0")

-------------------------------------------------------------------------------
-- Local variables
-------------------------------------------------------------------------------
LibQTipClick.Provider, LibQTipClick.Prototype, LibQTipClick.Base = LibQTip:CreateCellProvider(LibQTip.LabelProvider)

local cProvider, cPrototype, cBase = LibQTipClick.Provider, LibQTipClick.Prototype, LibQTipClick.Base

-------------------------------------------------------------------------------
-- Public library API
-------------------------------------------------------------------------------
local cellFunctions = {
	OnEnter = function(self)
			  if not self.highlight then
				  self.highlight = self:CreateTexture(nil, "BACKGROUND")
				  self.highlight:Hide()
			  end
			  self.highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
			  self.highlight:SetBlendMode("ADD")
			  self.highlight:SetAllPoints(self)
			  self.highlight:Show()
		  end,
	OnLeave = function(self)
			  self.highlight:ClearAllPoints()
			  self.highlight:Hide()
		  end,
	OnMouseDown = function(self)
			      DEFAULT_CHAT_FRAME:AddMessage("LibQTipClick: No function defined.")
		      end,
}

function cPrototype:InitializeCell()
	cBase.InitializeCell(self)
end

function cPrototype:SetupCell(tooltip, value, justification, font, r, g, b)
	local width, height = cBase.SetupCell(self, tooltip, value, justification, font, r, g, b)

	self:EnableMouse(true)

	local onEnter = tooltip.func.OnEnter or cellFunctions.OnEnter
	self:SetScript("OnEnter", onEnter)

	local onLeave = tooltip.func.OnLeave or cellFunctions.OnLeave
	self:SetScript("OnLeave", onLeave)

	local onMouseDown = tooltip.func.OnMouseDown or cellFunctions.OnMouseDown
	self:SetScript("OnMouseDown", onMouseDown)

	return width, height
end

function cPrototype:ReleaseCell()
	self:EnableMouse(false)
	self:SetScript("OnEnter", nil)
	self:SetScript("OnLeave", nil)
	self:SetScript("OnMouseDown", nil)
end

-------------------------------------------------------------------------------
-- LibQTip wrapper API
-------------------------------------------------------------------------------
function LibQTipClick:Acquire(key, ...)
	local tooltip = LibQTip:Acquire(key, ...)

	tooltip:EnableMouse(true)
	tooltip.func = {
		OnEnter = nil,
		OnLeave = nil,
		OnMouseDown = nil
	}
	return tooltip
end

function LibQTipClick:IsAcquired(key)
	return LibQTip:IsAcquired(key)
end

function LibQTipClick:Release(tooltip)
	tooltip:EnableMouse(false)
	tooltip.func = nil
	LibQTip:Release(tooltip)
end

function LibQTipClick:IterateTooltips()
	return LibQTip:IterateTooltips()
end

function LibQTipClick:CreateCellProvider(baseProvider)
	return LibQTip:CreateCellProvider(baseProvider)
end